from tlstrust import context
from .ccadb import __version__ as ccadb_version
from .linux import __version__ as linux_version
from .java import __version__ as java_version
from .certifi import __version__ as certifi_version
from .mintsifry_rossii import __version__ as russia_version
from .rustls import __version__ as rust_version
from .android_2_2 import __version__ as android2_2_version
from .android_2_3 import __version__ as android2_3_version
from .android_3 import __version__ as android3_version
from .android_4 import __version__ as android4_version
from .android_4_4 import __version__ as android4_4_version
from .android_7 import __version__ as android7_version
from .android_8 import __version__ as android8_version
from .android_9 import __version__ as android9_version
from .android_10 import __version__ as android10_version
from .android_11 import __version__ as android11_version
from .android_12 import __version__ as android12_version
from .android_13 import __version__ as android13_version
from .android_14 import __version__ as android14_version
from .android_latest import __version__ as android_version


__module__ = "tlstrust.stores"

VERSIONS = {
    context.CCADB: ccadb_version,
    context.JAVA_SRE: java_version,
    context.ANDROID: android_version,
    context.ANDROID_LATEST: android_version,
    context.GOOGLE_TRUST_SERVICES: android_version,
    context.ANDROID_FROYO: android2_2_version,
    context.ANDROID_GINGERBREAD: android2_3_version,
    context.ANDROID_HONEYCOMB: android3_version,
    context.ANDROID_ICE_CREAM_SANDWICH: android4_version,
    context.ANDROID_KITKAT: android4_4_version,
    context.ANDROID_NOUGAT: android7_version,
    context.ANDROID_OREO: android8_version,
    context.ANDROID_PIE: android9_version,
    context.ANDROID_QUINCE_TART: android10_version,
    context.ANDROID_RED_VELVET_CAKE: android11_version,
    context.ANDROID_SNOW_CONE: android12_version,
    context.ANDROID_TIRAMISU: android13_version,
    context.ANDROID_UPSIDE_DOWN_CAKE: android14_version,
    context.LINUX_ARCH: linux_version,
    context.PYTHON_CERTIFI: certifi_version,
    context.MINTSIFRY_ROSSII: russia_version,
    context.RUSTLS: rust_version,
    context.PYTHON: certifi_version,
    context.WINDOWS: ccadb_version,
    context.APPLE: ccadb_version,
    context.FIREFOX: ccadb_version,
    context.TOR: ccadb_version,
    context.CHROMIUM: ccadb_version,
    context.CHROME: ccadb_version,
    context.EDGE: ccadb_version,
    context.BRAVE: ccadb_version,
    context.OPERA: ccadb_version,
    context.VIVALDI: ccadb_version,
    context.SILK: ccadb_version,
    context.SAMSUNG: ccadb_version,
    context.YANDEX: russia_version,
    context.SAFARI: ccadb_version,
    context.PY_WINDOWS: ccadb_version,
    context.PY_LINUX: ccadb_version,
    context.PY_APPLE: ccadb_version,
    context.PY_CERTIFI: certifi_version,
    context.PY_URLLIB: certifi_version,
    context.PY_REQUESTS: certifi_version,
    context.PY_DJANGO: certifi_version,
    context.RUST_WINDOWS: rust_version,
    context.RUST_LINUX: rust_version,
    context.RUST_APPLE: rust_version,
    context.RUST_RUSTLS: rust_version,
    context.RUST_WEBPKI: rust_version,
    context.ERLANG_WINDOWS: ccadb_version,
    context.ERLANG_LINUX: ccadb_version,
    context.ERLANG_APPLE: ccadb_version,
    context.ERLANG_CERTIFI: certifi_version,
    context.GO_WINDOWS: ccadb_version,
    context.GO_LINUX: ccadb_version,
    context.GO_APPLE: ccadb_version,
    context.GO_CERTIFI: certifi_version,
    context.NODE_WINDOWS: ccadb_version,
    context.NODE_LINUX: ccadb_version,
    context.NODE_APPLE: ccadb_version,
    context.NODE_CERTIFI: certifi_version,
    context.RUBY_WINDOWS: ccadb_version,
    context.RUBY_LINUX: ccadb_version,
    context.RUBY_APPLE: ccadb_version,
    context.RUBY_CERTIFI: certifi_version,
}



__module__ = "tlstrust.context"

INVALID_CONTEXT = "context_type provided is invalid {}"
SOURCE_CCADB = 0
SOURCE_JAVA = 1
SOURCE_ANDROID = 3
SOURCE_LINUX = 4
SOURCE_RUSTLS = 5
SOURCE_CERTIFI = 101
SOURCE_RUSSIA = 201
PLATFORM_LINUX = SOURCE_LINUX
PLATFORM_JAVA = SOURCE_JAVA
PLATFORM_WINDOWS = SOURCE_CCADB
PLATFORM_APPLE = SOURCE_CCADB
PLATFORM_ANDROID = SOURCE_ANDROID
PLATFORM_ANDROID_LATEST = SOURCE_ANDROID
PLATFORM_ANDROID2_2 = 1302
PLATFORM_ANDROID2_3 = 2302
PLATFORM_ANDROID3 = 303
PLATFORM_ANDROID4 = 304
PLATFORM_ANDROID4_4 = 1304
PLATFORM_ANDROID7 = 307
PLATFORM_ANDROID8 = 308
PLATFORM_ANDROID9 = 309
PLATFORM_ANDROID10 = 310
PLATFORM_ANDROID11 = 311
PLATFORM_ANDROID12 = 312
PLATFORM_ANDROID13 = 313
PLATFORM_ANDROID14 = 314
PLATFORM_RUSSIA = SOURCE_RUSSIA
BROWSER_FIREFOX = SOURCE_CCADB
BROWSER_TOR_BROWSER = SOURCE_CCADB
BROWSER_CHROMIUM = SOURCE_CCADB
BROWSER_GOOGLE_CHROME = SOURCE_CCADB
BROWSER_MICROSOFT_EDGE = SOURCE_CCADB
BROWSER_BRAVE = SOURCE_CCADB
BROWSER_OPERA = SOURCE_CCADB
BROWSER_VIVALDI = SOURCE_CCADB
BROWSER_AMAZON_SILK = SOURCE_CCADB
BROWSER_SAMSUNG_INTERNET_BROWSER = SOURCE_CCADB
BROWSER_YANDEX_BROWSER = SOURCE_RUSSIA
BROWSER_SAFARI = SOURCE_CCADB
LANGUAGE_PYTHON_WINDOWS_SERVER = SOURCE_CCADB
LANGUAGE_PYTHON_LINUX_SERVER = SOURCE_LINUX
LANGUAGE_PYTHON_MACOS_SERVER = PLATFORM_APPLE
LANGUAGE_PYTHON_CERTIFI = SOURCE_CERTIFI
LANGUAGE_PYTHON_URLLIB = LANGUAGE_PYTHON_CERTIFI
LANGUAGE_PYTHON_REQUESTS = LANGUAGE_PYTHON_URLLIB
LANGUAGE_PYTHON_DJANGO = LANGUAGE_PYTHON_URLLIB
LANGUAGE_GO_WINDOWS_SERVER = SOURCE_CCADB
LANGUAGE_GO_LINUX_SERVER = SOURCE_LINUX
LANGUAGE_GO_MACOS_SERVER = PLATFORM_APPLE
LANGUAGE_GO_CERTIFI = SOURCE_CERTIFI
LANGUAGE_ERLANG_WINDOWS_SERVER = SOURCE_CCADB
LANGUAGE_ERLANG_LINUX_SERVER = SOURCE_LINUX
LANGUAGE_ERLANG_MACOS_SERVER = PLATFORM_APPLE
LANGUAGE_ERLANG_CERTIFI = SOURCE_CERTIFI
LANGUAGE_RUBY_WINDOWS_SERVER = SOURCE_CCADB
LANGUAGE_RUBY_LINUX_SERVER = SOURCE_LINUX
LANGUAGE_RUBY_MACOS_SERVER = PLATFORM_APPLE
LANGUAGE_RUBY_CERTIFI = SOURCE_CERTIFI
LANGUAGE_NODE_WINDOWS_SERVER = SOURCE_CCADB
LANGUAGE_NODE_LINUX_SERVER = SOURCE_LINUX
LANGUAGE_NODE_MACOS_SERVER = PLATFORM_APPLE
LANGUAGE_NODE_CERTIFI = SOURCE_CERTIFI
LANGUAGE_RUST_WINDOWS_SERVER = SOURCE_CCADB
LANGUAGE_RUST_LINUX_SERVER = SOURCE_LINUX
LANGUAGE_RUST_MACOS_SERVER = PLATFORM_APPLE
LANGUAGE_RUST_RUSTLS = SOURCE_RUSTLS
LANGUAGE_RUST_WEBPKI = SOURCE_RUSTLS
JAVA_SRE = "Java(TM) SE Runtime Environment"
CCADB = "Common Certificate Authority Database (CCADB)"
RUSTLS = "Rustls (curated CCADB)"
GOOGLE_TRUST_SERVICES = "Google Trust Services"
ANDROID = "Android (open source)"
ANDROID_LATEST = "Android (latest Google build)"
ANDROID_FROYO = "Android 2.2 (Froyo) 2010"
ANDROID_GINGERBREAD = "Android 2.3 (Gingerbread) 2010"
ANDROID_HONEYCOMB = "Android 3 (Honeycomb) 2011"
ANDROID_ICE_CREAM_SANDWICH = "Android 4 (Ice Cream Sandwich) 2011"
ANDROID_KITKAT = "Android 4.4 (KitKat) 2013"
ANDROID_NOUGAT = "Android 7 (Nougat) 2016"
ANDROID_OREO = "Android 8 (Oreo) 2017"
ANDROID_PIE = "Android 9 (Pie) 2018"
ANDROID_QUINCE_TART = "Android 10 (Quince Tart) 2019"
ANDROID_RED_VELVET_CAKE = "Android 11 (Red Velvet Cake) 2020"
ANDROID_SNOW_CONE = "Android 12 (Snow Cone) 2021"
ANDROID_TIRAMISU = "Android 13 (Tiramisu) 2022"
ANDROID_UPSIDE_DOWN_CAKE = "Android 14 (Upside Down Cake) 2023"
LINUX_ARCH = "Linux (Arch)"
MINTSIFRY_ROSSII = "MinTsifry Rossii"
PYTHON_CERTIFI = "Certifi (Python module for Certificate Authority Certificates)"
PYTHON = "Python 3.10"
WINDOWS = "Microsoft Windows"
APPLE = "Apple devices"
FIREFOX = "Mozilla Firefox"
TOR = "Tor Web Browser"
CHROMIUM = "Chromium Browser"
CHROME = "Google Chrome"
EDGE = "Microsoft Edge"
BRAVE = "Brave Browser"
OPERA = "Opera Browser"
VIVALDI = "Vivaldi Browser"
SILK = "Amazon Silk"
SAMSUNG = "Samsung Internet"
YANDEX = "Yandex"
SAFARI = "Apple Safari"
PY_WINDOWS = "Python built-in https module on Windows"
PY_LINUX = "Python built-in https module on Linux"
PY_APPLE = "Python built-in https module on Apple"
PY_CERTIFI = "certifi (Python module)"
PY_URLLIB = "urllib (Python module)"
PY_REQUESTS = "requests (Python module)"
PY_DJANGO = "Django (Python module)"
GO_WINDOWS = "Go built-in https module on Windows"
GO_LINUX = "Go built-in https module on Linux"
GO_APPLE = "Go built-in https module on Apple"
GO_CERTIFI = "gocertifi (Go module)"
ERLANG_WINDOWS = "Erlang built-in https module on Windows"
ERLANG_LINUX = "Erlang built-in https module on Linux"
ERLANG_APPLE = "Erlang built-in https module on Apple"
ERLANG_CERTIFI = "certifi (Erlang library)"
RUBY_WINDOWS = "Ruby built-in https module on Windows"
RUBY_LINUX = "Ruby built-in https module on Linux"
RUBY_APPLE = "Ruby built-in https module on Apple"
RUBY_CERTIFI = "certifi (Ruby gem)"
NODE_WINDOWS = "Node.js built-in https module on Windows"
NODE_LINUX = "Node.js built-in https module on Linux"
NODE_APPLE = "Node.js built-in https module on Apple"
NODE_CERTIFI = "certifi (Node.js package)"
RUST_WINDOWS = "Rust using Rustls crate on Windows"
RUST_LINUX = "Rust using Rustls crate on Linux"
RUST_APPLE = "Rust using Rustls crate on Apple"
RUST_RUSTLS = "Rustls (Rust crate) using webpki-roots"
RUST_WEBPKI = "webpki (Rust crate)"

SHORT_LOOKUP = {
    JAVA_SRE: "Java SE",
    CCADB: "CCADB",
    RUSTLS: "Rust",
    GOOGLE_TRUST_SERVICES: "Google",
    ANDROID: "Android FOSS",
    ANDROID_LATEST: "Android latest",
    ANDROID_FROYO: "Android 2.2",
    ANDROID_GINGERBREAD: "Android 2.3",
    ANDROID_HONEYCOMB: "Android 3",
    ANDROID_ICE_CREAM_SANDWICH: "Android 4",
    ANDROID_KITKAT: "Android 4.4",
    ANDROID_NOUGAT: "Android 7",
    ANDROID_OREO: "Android 8",
    ANDROID_PIE: "Android 9",
    ANDROID_QUINCE_TART: "Android 10",
    ANDROID_RED_VELVET_CAKE: "Android 11",
    ANDROID_SNOW_CONE: "Android 12",
    ANDROID_TIRAMISU: "Android 13",
    ANDROID_UPSIDE_DOWN_CAKE: "Android 14",
    LINUX_ARCH: "Linux",
    MINTSIFRY_ROSSII: "Russian",
    PYTHON_CERTIFI: "Certifi",
    WINDOWS: "Windows",
    APPLE: "Apple",
    FIREFOX: "Firefox",
    TOR: "Tor",
    CHROMIUM: "Chromium",
    CHROME: "Chrome",
    EDGE: "Edge",
    BRAVE: "Brave",
    OPERA: "Opera",
    VIVALDI: "Vivaldi",
    SILK: "Silk",
    SAMSUNG: "Samsung",
    YANDEX: "Yandex",
    SAFARI: "Safari",
    PY_WINDOWS: "Python on Windows",
    PY_LINUX: "Python on Linux",
    PY_APPLE: "Python on Apple",
    PY_CERTIFI: "certifi",
    PY_URLLIB: "urllib",
    PY_REQUESTS: "requests",
    PY_DJANGO: "Django",
    ERLANG_WINDOWS: "Erlang on Windows",
    ERLANG_LINUX: "Erlang on Linux",
    ERLANG_APPLE: "Erlang on Apple",
    ERLANG_CERTIFI: "erlang-certifi",
    GO_WINDOWS: "Go on Windows",
    GO_LINUX: "Go on Linux",
    GO_APPLE: "Go on Apple",
    GO_CERTIFI: "gocertifi",
    RUBY_WINDOWS: "Ruby on Windows",
    RUBY_LINUX: "Ruby on Linux",
    RUBY_APPLE: "Ruby on Apple",
    RUBY_CERTIFI: "ruby-certifi",
    NODE_WINDOWS: "Node.js on Windows",
    NODE_LINUX: "Node.js on Linux",
    NODE_APPLE: "Node.js on Apple",
    NODE_CERTIFI: "node-certifi",
    RUST_WINDOWS: "Rust on Windows",
    RUST_LINUX: "Rust on Linux",
    RUST_APPLE: "Rust on Apple",
    RUST_RUSTLS: "Rustls crate",
    RUST_WEBPKI: "Web PKI crate",
}
STORES = {
    JAVA_SRE: SOURCE_JAVA,
    CCADB: SOURCE_CCADB,
    RUSTLS: SOURCE_RUSTLS,
    GOOGLE_TRUST_SERVICES: SOURCE_ANDROID,
    ANDROID_FROYO: PLATFORM_ANDROID2_2,
    ANDROID_GINGERBREAD: PLATFORM_ANDROID2_3,
    ANDROID_HONEYCOMB: PLATFORM_ANDROID3,
    ANDROID_ICE_CREAM_SANDWICH: PLATFORM_ANDROID4,
    ANDROID_KITKAT: PLATFORM_ANDROID4_4,
    ANDROID_NOUGAT: PLATFORM_ANDROID7,
    ANDROID_OREO: PLATFORM_ANDROID8,
    ANDROID_PIE: PLATFORM_ANDROID9,
    ANDROID_QUINCE_TART: PLATFORM_ANDROID10,
    ANDROID_RED_VELVET_CAKE: PLATFORM_ANDROID11,
    ANDROID_SNOW_CONE: PLATFORM_ANDROID12,
    ANDROID_TIRAMISU: PLATFORM_ANDROID13,
    ANDROID_UPSIDE_DOWN_CAKE: PLATFORM_ANDROID14,
    LINUX_ARCH: SOURCE_LINUX,
    LANGUAGE_PYTHON_CERTIFI: SOURCE_CERTIFI,
    MINTSIFRY_ROSSII: SOURCE_RUSSIA,
}
SOURCES = {
    CCADB: SOURCE_CCADB,
    RUSTLS: SOURCE_RUSTLS,
    JAVA_SRE: SOURCE_JAVA,
    GOOGLE_TRUST_SERVICES: SOURCE_ANDROID,
    LINUX_ARCH: SOURCE_LINUX,
    PYTHON_CERTIFI: SOURCE_CERTIFI,
    MINTSIFRY_ROSSII: SOURCE_RUSSIA,
}
PLATFORMS = {
    JAVA_SRE: PLATFORM_JAVA,
    WINDOWS: PLATFORM_WINDOWS,
    APPLE: PLATFORM_APPLE,
    ANDROID: PLATFORM_ANDROID,
    ANDROID_LATEST: PLATFORM_ANDROID_LATEST,
    ANDROID_FROYO: PLATFORM_ANDROID2_2,
    ANDROID_GINGERBREAD: PLATFORM_ANDROID2_3,
    ANDROID_HONEYCOMB: PLATFORM_ANDROID3,
    ANDROID_ICE_CREAM_SANDWICH: PLATFORM_ANDROID4,
    ANDROID_KITKAT: PLATFORM_ANDROID4_4,
    ANDROID_NOUGAT: PLATFORM_ANDROID7,
    ANDROID_OREO: PLATFORM_ANDROID8,
    ANDROID_PIE: PLATFORM_ANDROID9,
    ANDROID_QUINCE_TART: PLATFORM_ANDROID10,
    ANDROID_RED_VELVET_CAKE: PLATFORM_ANDROID11,
    ANDROID_SNOW_CONE: PLATFORM_ANDROID12,
    ANDROID_TIRAMISU: PLATFORM_ANDROID13,
    ANDROID_UPSIDE_DOWN_CAKE: PLATFORM_ANDROID14,
    MINTSIFRY_ROSSII: PLATFORM_RUSSIA,
}
BROWSERS = {
    FIREFOX: BROWSER_FIREFOX,
    TOR: BROWSER_TOR_BROWSER,
    CHROMIUM: BROWSER_CHROMIUM,
    CHROME: BROWSER_GOOGLE_CHROME,
    EDGE: BROWSER_MICROSOFT_EDGE,
    BRAVE: BROWSER_BRAVE,
    OPERA: BROWSER_OPERA,
    VIVALDI: BROWSER_VIVALDI,
    SILK: BROWSER_AMAZON_SILK,
    SAMSUNG: BROWSER_SAMSUNG_INTERNET_BROWSER,
    YANDEX: BROWSER_YANDEX_BROWSER,
    SAFARI: BROWSER_SAFARI,
}
LANGUAGES = {
    PY_WINDOWS: LANGUAGE_PYTHON_WINDOWS_SERVER,
    PY_LINUX: LANGUAGE_PYTHON_LINUX_SERVER,
    PY_APPLE: LANGUAGE_PYTHON_MACOS_SERVER,
    PY_CERTIFI: LANGUAGE_PYTHON_CERTIFI,
    PY_URLLIB: LANGUAGE_PYTHON_URLLIB,
    PY_REQUESTS: LANGUAGE_PYTHON_REQUESTS,
    PY_DJANGO: LANGUAGE_PYTHON_DJANGO,
    GO_WINDOWS: LANGUAGE_GO_WINDOWS_SERVER,
    GO_LINUX: LANGUAGE_GO_LINUX_SERVER,
    GO_APPLE: LANGUAGE_GO_MACOS_SERVER,
    GO_CERTIFI: LANGUAGE_GO_CERTIFI,
    ERLANG_WINDOWS: LANGUAGE_ERLANG_WINDOWS_SERVER,
    ERLANG_LINUX: LANGUAGE_ERLANG_LINUX_SERVER,
    ERLANG_APPLE: LANGUAGE_ERLANG_MACOS_SERVER,
    ERLANG_CERTIFI: LANGUAGE_ERLANG_CERTIFI,
    RUBY_WINDOWS: LANGUAGE_RUBY_WINDOWS_SERVER,
    RUBY_LINUX: LANGUAGE_RUBY_LINUX_SERVER,
    RUBY_APPLE: LANGUAGE_RUBY_MACOS_SERVER,
    RUBY_CERTIFI: LANGUAGE_RUBY_CERTIFI,
    NODE_WINDOWS: LANGUAGE_NODE_WINDOWS_SERVER,
    NODE_LINUX: LANGUAGE_NODE_LINUX_SERVER,
    NODE_APPLE: LANGUAGE_NODE_MACOS_SERVER,
    NODE_CERTIFI: LANGUAGE_NODE_CERTIFI,
    RUST_WINDOWS: LANGUAGE_RUST_WINDOWS_SERVER,
    RUST_LINUX: LANGUAGE_RUST_LINUX_SERVER,
    RUST_APPLE: LANGUAGE_RUST_MACOS_SERVER,
    RUST_RUSTLS: LANGUAGE_RUST_RUSTLS,
    RUST_WEBPKI: LANGUAGE_RUST_WEBPKI,
}



import sys
import logging
from datetime import datetime
from OpenSSL.crypto import X509
from .util import (
    InvalidChainError,
    get_cn_or_org,
    valid_context_type,
    get_certificate_from_store,
    match_certificate,
    get_leaf,
    build_chains,
    get_store_result_text,
)
from .context import *  # noqa: F403
from .stores.android_2_2 import (
    UNTRUSTED as ANDROID2_2_UNTRUSTED,
    PEM_FILES as ANDROID2_2_PEM_FILES,
)
from .stores.android_2_3 import (
    UNTRUSTED as ANDROID2_3_UNTRUSTED,
    PEM_FILES as ANDROID2_3_PEM_FILES,
)
from .stores.android_3 import (
    UNTRUSTED as ANDROID3_UNTRUSTED,
    PEM_FILES as ANDROID3_PEM_FILES,
)
from .stores.android_4_4 import (
    UNTRUSTED as ANDROID4_4_UNTRUSTED,
    PEM_FILES as ANDROID4_4_PEM_FILES,
)
from .stores.android_4 import (
    UNTRUSTED as ANDROID4_UNTRUSTED,
    PEM_FILES as ANDROID4_PEM_FILES,
)
from .stores.android_7 import (
    UNTRUSTED as ANDROID7_UNTRUSTED,
    PEM_FILES as ANDROID7_PEM_FILES,
)
from .stores.android_8 import (
    UNTRUSTED as ANDROID8_UNTRUSTED,
    PEM_FILES as ANDROID8_PEM_FILES,
)
from .stores.android_9 import (
    UNTRUSTED as ANDROID9_UNTRUSTED,
    PEM_FILES as ANDROID9_PEM_FILES,
)
from .stores.android_10 import (
    UNTRUSTED as ANDROID10_UNTRUSTED,
    PEM_FILES as ANDROID10_PEM_FILES,
)
from .stores.android_11 import (
    UNTRUSTED as ANDROID11_UNTRUSTED,
    PEM_FILES as ANDROID11_PEM_FILES,
)
from .stores.android_12 import (
    UNTRUSTED as ANDROID12_UNTRUSTED,
    PEM_FILES as ANDROID12_PEM_FILES,
)
from .stores.android_13 import (
    UNTRUSTED as ANDROID13_UNTRUSTED,
    PEM_FILES as ANDROID13_PEM_FILES,
)
from .stores.android_14 import (
    UNTRUSTED as ANDROID14_UNTRUSTED,
    PEM_FILES as ANDROID14_PEM_FILES,
)
from .stores.android_latest import (
    UNTRUSTED as ANDROID_UNTRUSTED,
    PEM_FILES as ANDROID_PEM_FILES,
)
from .stores.ccadb import UNTRUSTED as CCADB_UNTRUSTED, PEM_FILES as CCADB_PEM_FILES
from .stores.java import UNTRUSTED as JAVA_UNTRUSTED, PEM_FILES as JAVA_PEM_FILES
from .stores.linux import UNTRUSTED as LINUX_UNTRUSTED, PEM_FILES as LINUX_PEM_FILES
from .stores.certifi import (
    UNTRUSTED as CERTIFI_UNTRUSTED,
    PEM_FILES as CERTIFI_PEM_FILES,
)
from .stores.mintsifry_rossii import (
    UNTRUSTED as RUSSIA_UNTRUSTED,
    PEM_FILES as RUSSIA_PEM_FILES,
)

__module__ = "tlstrust"
__version__ = "2.5.3"

assert sys.version_info >= (3, 9), "Requires Python 3.9 or newer"

logger = logging.getLogger(__name__)


class TrustStore:
    key_identifier: str

    def __init__(self, authority_key_identifier: str) -> bool:
        if not isinstance(authority_key_identifier, str):
            raise TypeError(
                f"authority_key_identifier type {type(authority_key_identifier)} not supported, expected str"
            )
        # used for Root CA matching, SKI is authoritative
        self.key_identifier = authority_key_identifier
        for _, ctx in SOURCES.items():
            if self.exists(context_type=ctx):
                break

    def to_dict(self) -> dict:
        contexts = {**SOURCES, **PLATFORMS, **BROWSERS, **LANGUAGES}
        subject_common_name = get_cn_or_org(self.certificate)
        data = {
            "trust_stores": [],
            "_metadata": {
                "last_updated": datetime.utcnow().replace(microsecond=0).isoformat(),
                "certificate_not_valid_after": self.certificate.to_cryptography().not_valid_after,
                "certificate_issuer": subject_common_name,
                "certificate_issuer_ski": self.key_identifier,
            },
        }
        for name, is_trusted in self.all_results.items():
            ctx = None
            for _name, _ctx in contexts.items():
                if name == _name:
                    ctx = _ctx
                    break
            result = {}
            result["name"] = name
            result["is_trusted"] = is_trusted
            try:
                result["exists"] = isinstance(self.certificate, X509)
                result["expired"] = self.expired_in_store(ctx)
            except FileExistsError:
                result["exists"] = False
            result["description"] = get_store_result_text(**result)
            data["trust_stores"].append(result)
        return data

    @property
    def all_results(self) -> dict:
        results = {}
        contexts = {**SOURCES, **PLATFORMS, **BROWSERS, **LANGUAGES}
        for name, ctx in contexts.items():
            results[name] = self.check_trust(ctx)
        return results

    @property
    def certificate(self) -> X509:
        certificate = None
        for _, context_type in STORES.items():
            try:
                certificate = get_certificate_from_store(
                    self.key_identifier, context_type
                )
            except FileExistsError:
                continue
            if isinstance(certificate, X509):
                break
        return certificate

    @property
    def ccadb(self) -> bool:
        return (
            self.key_identifier not in CCADB_UNTRUSTED
            and self.key_identifier in CCADB_PEM_FILES.keys()
        )

    @property
    def java(self) -> bool:
        return (
            self.key_identifier not in JAVA_UNTRUSTED
            and self.key_identifier in JAVA_PEM_FILES.keys()
        )

    @property
    def android(self) -> bool:
        untrusted = set(
            ANDROID_UNTRUSTED
            + ANDROID2_2_UNTRUSTED
            + ANDROID2_3_UNTRUSTED
            + ANDROID3_UNTRUSTED
            + ANDROID4_UNTRUSTED
            + ANDROID4_4_UNTRUSTED
            + ANDROID7_UNTRUSTED
            + ANDROID8_UNTRUSTED
            + ANDROID9_UNTRUSTED
            + ANDROID10_UNTRUSTED
            + ANDROID11_UNTRUSTED
            + ANDROID12_UNTRUSTED
            + ANDROID13_UNTRUSTED
            + ANDROID14_UNTRUSTED
        )
        files = (
            ANDROID_PEM_FILES
            | ANDROID2_2_PEM_FILES
            | ANDROID2_3_PEM_FILES
            | ANDROID3_PEM_FILES
            | ANDROID4_PEM_FILES
            | ANDROID4_4_PEM_FILES
            | ANDROID7_PEM_FILES
            | ANDROID8_PEM_FILES
            | ANDROID9_PEM_FILES
            | ANDROID10_PEM_FILES
            | ANDROID11_PEM_FILES
            | ANDROID12_PEM_FILES
            | ANDROID13_PEM_FILES
            | ANDROID14_PEM_FILES
        )
        return self.key_identifier not in untrusted and self.key_identifier in set(
            files.keys()
        )

    @property
    def android_latest(self) -> bool:
        return (
            self.key_identifier not in ANDROID_UNTRUSTED
            and self.key_identifier in ANDROID_PEM_FILES.keys()
        )

    @property
    def android14(self) -> bool:
        return (
            self.key_identifier not in ANDROID14_UNTRUSTED
            and self.key_identifier in ANDROID14_PEM_FILES.keys()
        )

    @property
    def android13(self) -> bool:
        return (
            self.key_identifier not in ANDROID13_UNTRUSTED
            and self.key_identifier in ANDROID13_PEM_FILES.keys()
        )

    @property
    def android12(self) -> bool:
        return (
            self.key_identifier not in ANDROID12_UNTRUSTED
            and self.key_identifier in ANDROID12_PEM_FILES.keys()
        )

    @property
    def android11(self) -> bool:
        return (
            self.key_identifier not in ANDROID11_UNTRUSTED
            and self.key_identifier in ANDROID11_PEM_FILES.keys()
        )

    @property
    def android10(self) -> bool:
        return (
            self.key_identifier not in ANDROID10_UNTRUSTED
            and self.key_identifier in ANDROID10_PEM_FILES.keys()
        )

    @property
    def android9(self) -> bool:
        return (
            self.key_identifier not in ANDROID9_UNTRUSTED
            and self.key_identifier in ANDROID9_PEM_FILES.keys()
        )

    @property
    def android8(self) -> bool:
        return (
            self.key_identifier not in ANDROID8_UNTRUSTED
            and self.key_identifier in ANDROID8_PEM_FILES.keys()
        )

    @property
    def android7(self) -> bool:
        return (
            self.key_identifier not in ANDROID7_UNTRUSTED
            and self.key_identifier in ANDROID7_PEM_FILES.keys()
        )

    @property
    def android4_4(self) -> bool:
        return (
            self.key_identifier not in ANDROID4_4_UNTRUSTED
            and self.key_identifier in ANDROID4_4_PEM_FILES.keys()
        )

    @property
    def android4(self) -> bool:
        return (
            self.key_identifier not in ANDROID4_UNTRUSTED
            and self.key_identifier in ANDROID4_PEM_FILES.keys()
        )

    @property
    def android3(self) -> bool:
        return (
            self.key_identifier not in ANDROID3_UNTRUSTED
            and self.key_identifier in ANDROID3_PEM_FILES.keys()
        )

    @property
    def android2_3(self) -> bool:
        return (
            self.key_identifier not in ANDROID2_3_UNTRUSTED
            and self.key_identifier in ANDROID2_3_PEM_FILES.keys()
        )

    @property
    def android2_2(self) -> bool:
        return (
            self.key_identifier not in ANDROID2_2_UNTRUSTED
            and self.key_identifier in ANDROID2_2_PEM_FILES.keys()
        )

    @property
    def linux(self) -> bool:
        return (
            self.key_identifier not in LINUX_UNTRUSTED
            and self.key_identifier in LINUX_PEM_FILES.keys()
        )

    @property
    def certifi(self) -> bool:
        return (
            self.key_identifier not in CERTIFI_UNTRUSTED
            and self.key_identifier in CERTIFI_PEM_FILES.keys()
        )

    @property
    def russia(self) -> bool:
        return (
            self.key_identifier not in RUSSIA_UNTRUSTED
            and self.key_identifier in RUSSIA_PEM_FILES.keys()
        )

    @property
    def is_trusted(self) -> bool:
        return all([self.ccadb, self.android, self.linux, self.certifi, self.java])

    def exists(self, context_type: int) -> bool:
        if not valid_context_type(context_type):
            raise AttributeError(INVALID_CONTEXT.format(context_type))

        if (
            context_type == SOURCE_CCADB
            and self.key_identifier in CCADB_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, SOURCE_CCADB),
            )
        if context_type == SOURCE_JAVA and self.key_identifier in JAVA_PEM_FILES.keys():
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, SOURCE_JAVA),
            )
        if (
            context_type == SOURCE_ANDROID
            and self.key_identifier in ANDROID_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, SOURCE_ANDROID),
            )
        if (
            context_type == SOURCE_LINUX
            and self.key_identifier in LINUX_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, SOURCE_LINUX),
            )
        if (
            context_type == SOURCE_RUSSIA
            and self.key_identifier in RUSSIA_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, SOURCE_RUSSIA),
            )
        if (
            context_type == SOURCE_CERTIFI
            and self.key_identifier in CERTIFI_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, SOURCE_CERTIFI),
            )
        if (
            context_type == PLATFORM_ANDROID14
            and self.key_identifier in ANDROID14_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, PLATFORM_ANDROID14),
            )
        if (
            context_type == PLATFORM_ANDROID13
            and self.key_identifier in ANDROID13_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, PLATFORM_ANDROID13),
            )
        if (
            context_type == PLATFORM_ANDROID12
            and self.key_identifier in ANDROID12_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, PLATFORM_ANDROID12),
            )
        if (
            context_type == PLATFORM_ANDROID11
            and self.key_identifier in ANDROID11_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, PLATFORM_ANDROID11),
            )
        if (
            context_type == PLATFORM_ANDROID10
            and self.key_identifier in ANDROID10_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, PLATFORM_ANDROID10),
            )
        if (
            context_type == PLATFORM_ANDROID9
            and self.key_identifier in ANDROID9_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, PLATFORM_ANDROID9),
            )
        if (
            context_type == PLATFORM_ANDROID8
            and self.key_identifier in ANDROID8_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, PLATFORM_ANDROID8),
            )
        if (
            context_type == PLATFORM_ANDROID7
            and self.key_identifier in ANDROID7_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, PLATFORM_ANDROID7),
            )
        if (
            context_type == PLATFORM_ANDROID4_4
            and self.key_identifier in ANDROID4_4_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, PLATFORM_ANDROID7),
            )
        if (
            context_type == PLATFORM_ANDROID4
            and self.key_identifier in ANDROID4_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, PLATFORM_ANDROID7),
            )
        if (
            context_type == PLATFORM_ANDROID3
            and self.key_identifier in ANDROID3_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, PLATFORM_ANDROID7),
            )
        if (
            context_type == PLATFORM_ANDROID2_3
            and self.key_identifier in ANDROID2_3_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, PLATFORM_ANDROID7),
            )
        if (
            context_type == PLATFORM_ANDROID2_2
            and self.key_identifier in ANDROID2_2_PEM_FILES.keys()
        ):
            return match_certificate(
                self.key_identifier,
                get_certificate_from_store(self.key_identifier, PLATFORM_ANDROID7),
            )
        return False

    def expired_in_store(self, context_type: int) -> bool:
        if not valid_context_type(context_type):
            raise AttributeError(INVALID_CONTEXT.format(context_type))
        if not self.exists(context_type=context_type):
            raise FileExistsError("Certificate does not exist")
        return get_certificate_from_store(
            self.key_identifier, context_type=context_type
        ).has_expired()

    def check_trust(self, context_type: int = None) -> bool:
        if context_type is not None and not isinstance(context_type, int):
            raise TypeError(
                f"context type {type(context_type)} not supported, expected int"
            )
        if not valid_context_type(context_type):
            raise AttributeError(INVALID_CONTEXT.format(context_type))

        if context_type == SOURCE_CCADB:
            return self.ccadb
        if context_type == SOURCE_JAVA:
            return self.java
        if context_type == SOURCE_ANDROID:
            return self.android
        if context_type == PLATFORM_ANDROID14:
            return self.android14
        if context_type == PLATFORM_ANDROID13:
            return self.android13
        if context_type == PLATFORM_ANDROID12:
            return self.android12
        if context_type == PLATFORM_ANDROID11:
            return self.android11
        if context_type == PLATFORM_ANDROID10:
            return self.android10
        if context_type == PLATFORM_ANDROID9:
            return self.android9
        if context_type == PLATFORM_ANDROID8:
            return self.android8
        if context_type == PLATFORM_ANDROID7:
            return self.android7
        if context_type == PLATFORM_ANDROID4_4:
            return self.android4_4
        if context_type == PLATFORM_ANDROID4:
            return self.android4
        if context_type == PLATFORM_ANDROID3:
            return self.android3
        if context_type == PLATFORM_ANDROID2_3:
            return self.android2_3
        if context_type == PLATFORM_ANDROID2_2:
            return self.android2_2
        if context_type == SOURCE_LINUX:
            return self.linux
        if context_type == SOURCE_CERTIFI:
            return self.certifi
        if context_type == SOURCE_RUSSIA:
            return self.russia

        return self.is_trusted


def trust_stores_from_chain(certificates: list[X509]) -> list[TrustStore]:
    leaf = get_leaf(certificates)
    if not isinstance(leaf, X509):
        raise InvalidChainError(
            "certificate chain is empty or missing a server leaf certificate"
        )
    chain = build_chains(leaf, certificates)
    return [TrustStore(root.get("ski")) for _, root in chain.items()]



from setuptools import setup, find_packages

setup(
    name="tlstrust",
    version="2.5.3",
    author='Christopher Langton',
    author_email='chris@langton.cloud',
    description="Utilities that assist with trust relationship checking of X.509 Certificates for various end-user devices with disparate root trust stores.",
    long_description="""
# tlstrust

Utilities that assist with trust relationship checking of X.509 Certificates for various end-user devices with disparate root trust stores.

![tlstrust cli](https://gitlab.com/trivialsec/tlstrust/-/raw/5052faa1d1a5fbf21dec6107622cffc58359762d/docs/images/tlstrust.jpg)

## [Documentation](https://gitlab.com/trivialsec/tlstrust/-/blob/main/docs/0.index.md)

On the command-line:

```sh
tlstrust --help
```

produces:

```
usage: tlstrust [-h] [-C CLIENT_PEM] [--disable-sni] [-v] [-vv] [-vvv] [-vvvv] [--version] [targets ...]

positional arguments:
  targets               All unnamed arguments are hosts (and ports) targets to test. ~$ tlstrust apple.com:443 github.io
                        localhost:3000

options:
  -h, --help            show this help message and exit
  -C CLIENT_PEM, --client-pem CLIENT_PEM
                        path to PEM encoded client certificate, url or file path accepted
  --disable-sni         Do not negotiate SNI using INDA encoded host
  -v, --errors-only     set logging level to ERROR (default CRITICAL)
  -vv, --warning        set logging level to WARNING (default CRITICAL)
  -vvv, --info          set logging level to INFO (default CRITICAL)
  -vvvv, --debug        set logging level to DEBUG (default CRITICAL)
  --version
```

In your app you can:

```py
import os
from pathlib import Path
from OpenSSL.crypto import FILETYPE_ASN1
from tlstrust import TrustStore

der = Path(os.path.join(os.path.dirname(__file__), "cacert.der")).read_bytes()
trust_store = TrustStore(FILETYPE_ASN1, der)
print(trust_store.check_trust())
```

# Platform specific checking

```py
all_trusted = trust_store.check_trust()
assert all_trusted is True
assert trust_store.android
assert trust_store.linux
assert trust_store.ccadb # Windows, Mozilla, and Apple (from December 1st 2021)
assert trust_store.java
assert trust_store.certifi
```

## Basic usage

Using CCADB for demonstration purposes (includes Apple, Microsoft, and Mozilla)

```py
from tlstrust.context import SOURCE_CCADB

assert trust_store.exists(SOURCE_CCADB)
assert trust_store.expired_in_store(SOURCE_CCADB)
assert trust_store.get_certificate_from_store(SOURCE_CCADB)
assert trust_store.check_trust(SOURCE_CCADB)
```

## Other Platforms

```py
from tlstrust.context import PLATFORM_ANDROID
from tlstrust.context import PLATFORM_JAVA
from tlstrust.context import PLATFORM_LINUX
from tlstrust.context import PLATFORM_APPLE
```

## Apple (before CCADB)

Apple (legacy) Trust Store support exists in earlier versions of `tlstrust`, it was removed in version `2.0.0` so installing prior versions will allow you to access this functionality.

## Android versions

```py
from tlstrust.context import PLATFORM_ANDROID2_2
from tlstrust.context import PLATFORM_ANDROID2_3
from tlstrust.context import PLATFORM_ANDROID3
from tlstrust.context import PLATFORM_ANDROID4
from tlstrust.context import PLATFORM_ANDROID4_4
from tlstrust.context import PLATFORM_ANDROID7
from tlstrust.context import PLATFORM_ANDROID8
from tlstrust.context import PLATFORM_ANDROID9
from tlstrust.context import PLATFORM_ANDROID10
from tlstrust.context import PLATFORM_ANDROID11
from tlstrust.context import PLATFORM_ANDROID12
```

# Browser Trust Stores

```py
from tlstrust.context import BROWSER_AMAZON_SILK, BROWSER_SAMSUNG_INTERNET_BROWSER, BROWSER_GOOGLE_CHROME, BROWSER_CHROMIUM, BROWSER_FIREFOX, BROWSER_BRAVE, BROWSER_SAFARI, BROWSER_MICROSOFT_EDGE, BROWSER_YANDEX_BROWSER, BROWSER_OPERA, BROWSER_VIVALDI, BROWSER_TOR_BROWSER

assert trust_store.check_trust(BROWSER_AMAZON_SILK)
assert trust_store.check_trust(BROWSER_SAMSUNG_INTERNET_BROWSER)
assert trust_store.check_trust(BROWSER_GOOGLE_CHROME)
assert trust_store.check_trust(BROWSER_CHROMIUM)
assert trust_store.check_trust(BROWSER_FIREFOX)
assert trust_store.check_trust(BROWSER_BRAVE)
assert trust_store.check_trust(BROWSER_SAFARI)
assert trust_store.check_trust(BROWSER_MICROSOFT_EDGE)
assert trust_store.check_trust(BROWSER_YANDEX_BROWSER)
assert trust_store.check_trust(BROWSER_OPERA)
assert trust_store.check_trust(BROWSER_VIVALDI)
assert trust_store.check_trust(BROWSER_TOR_BROWSER)
```

# Programming Language Trust (Microservice architecture and APIs)

Python:

```py
from tlstrust.context import LANGUAGE_PYTHON_WINDOWS_SERVER, LANGUAGE_PYTHON_LINUX_SERVER, LANGUAGE_PYTHON_MACOS_SERVER, LANGUAGE_PYTHON_CERTIFI, LANGUAGE_PYTHON_URLLIB, LANGUAGE_PYTHON_REQUESTS, LANGUAGE_PYTHON_DJANGO

assert trust_store.check_trust(LANGUAGE_PYTHON_WINDOWS_SERVER)
assert trust_store.check_trust(LANGUAGE_PYTHON_LINUX_SERVER)
assert trust_store.check_trust(LANGUAGE_PYTHON_MACOS_SERVER)
assert trust_store.check_trust(LANGUAGE_PYTHON_CERTIFI)
assert trust_store.check_trust(LANGUAGE_PYTHON_URLLIB)
assert trust_store.check_trust(LANGUAGE_PYTHON_REQUESTS)
assert trust_store.check_trust(LANGUAGE_PYTHON_DJANGO)
```

## [Change Log](https://gitlab.com/trivialsec/tlstrust/-/blob/main/docs/z.change-log.md)
    """,
    long_description_content_type="text/markdown",
    url="https://gitlab.com/trivialsec/tlstrust",
    project_urls={
        "Source": "https://gitlab.com/trivialsec/tlstrust",
        "Documentation": "https://gitlab.com/trivialsec/tlstrust/-/blob/main/docs/0.index.md",
        "Tracker": "https://gitlab.com/trivialsec/tlstrust/-/issues",
    },
    classifiers=[
        "Operating System :: OS Independent",
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
    ],
    include_package_data=True,
    install_requires=[
        'certifi',
        'cryptography==35.0.0',
        'asn1crypto==1.4.0',
        'pyOpenSSL==21.0.0',
        'rich==12.0.0',
        'validators==0.18.2',
        'idna==3.3'
    ],
    entry_points = {
        'console_scripts': ['tlstrust=tlstrust.cli:cli'],
    },
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    python_requires=">=3.9",
    options={"bdist_wheel": {"universal": "1"}},
)
